@@ -3,7 +3,7 @@
    DHCP/BOOTP Relay Agent. */
 
 /*
- * Copyright(c) 2004-2015 by Internet Systems Consortium, Inc.("ISC")
+ * Copyright(c) 2004-2016 by Internet Systems Consortium, Inc.("ISC")
  * Copyright(c) 1997-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -30,6 +30,7 @@
 #include <syslog.h>
 #include <signal.h>
 #include <sys/time.h>
+#include <isc/file.h>
 
 TIME default_lease_time = 43200; /* 12 hours... */
 TIME max_lease_time = 86400; /* 24 hours... */
@@ -60,6 +61,7 @@ int server_packets_relayed = 0;	/* Packets relayed from server to client. */
 int client_packet_errors = 0;	/* Errors sending packets to clients. */
 
 int add_agent_options = 0;	/* If nonzero, add relay agent options. */
+int add_rfc3527_suboption = 0;	/* If nonzero, add RFC3527 link selection sub-option. */
 
 int agent_option_errors = 0;    /* Number of packets forwarded without
 				   agent options because there was no room. */
@@ -99,6 +101,8 @@ struct server_list {
 	struct sockaddr_in to;
 } *servers;
 
+struct interface_info *uplink = NULL;
+
 #ifdef DHCPv6
 struct stream_list {
 	struct stream_list *next;
@@ -132,23 +136,30 @@ static int strip_relay_agent_options(struct interface_info *,
 				     struct interface_info **,
 				     struct dhcp_packet *, unsigned);
 
+static void request_v4_interface(const char* name, int flags);
+
 static const char copyright[] =
-"Copyright 2004-2015 Internet Systems Consortium.";
+"Copyright 2004-2016 Internet Systems Consortium.";
 static const char arr[] = "All rights reserved.";
 static const char message[] =
 "Internet Systems Consortium DHCP Relay Agent";
 static const char url[] =
 "For info, please visit https://www.isc.org/software/dhcp/";
 
+char *progname;
+
 #ifdef DHCPv6
 #define DHCRELAY_USAGE \
-"Usage: dhcrelay [-4] [-d] [-q] [-a] [-D]\n"\
+"Usage: %s [-4] [-d] [-q] [-a] [-D]\n"\
 "                     [-A <length>] [-c <hops>] [-p <port>]\n" \
 "                     [-pf <pid-file>] [--no-pid]\n"\
 "                     [-m append|replace|forward|discard]\n" \
 "                     [-i interface0 [ ... -i interfaceN]\n" \
+"                     [-iu interface0 [ ... -iu interfaceN]\n" \
+"                     [-id interface0 [ ... -id interfaceN]\n" \
+"                     [-U interface]\n" \
 "                     server0 [ ... serverN]\n\n" \
-"       dhcrelay -6   [-d] [-q] [-I] [-c <hops>] [-p <port>]\n" \
+"       %s -6   [-d] [-q] [-I] [-c <hops>] [-p <port>]\n" \
 "                     [-pf <pid-file>] [--no-pid]\n" \
 "                     [-s <subscriber-id>]\n" \
 "                     -l lower0 [ ... -l lowerN]\n" \
@@ -157,15 +168,52 @@ static const char url[] =
 "       upper (server link): [address%%]interface"
 #else
 #define DHCRELAY_USAGE \
-"Usage: dhcrelay [-d] [-q] [-a] [-D] [-A <length>] [-c <hops>] [-p <port>]\n" \
+"Usage: %s [-d] [-q] [-a] [-D] [-A <length>] [-c <hops>] [-p <port>]\n" \
 "                [-pf <pid-file>] [--no-pid]\n" \
 "                [-m append|replace|forward|discard]\n" \
 "                [-i interface0 [ ... -i interfaceN]\n" \
+"                [-iu interface0 [ ... -iu interfaceN]\n" \
+"                [-id interface0 [ ... -id interfaceN]\n" \
+"                [-U interface]\n" \
 "                server0 [ ... serverN]\n\n"
 #endif
 
-static void usage() {
-	log_fatal(DHCRELAY_USAGE);
+/*!
+ *
+ * \brief Print the generic usage message
+ *
+ * If the user has provided an incorrect command line print out
+ * the description of the command line.  The arguments provide
+ * a way for the caller to request more specific information about
+ * the error be printed as well.  Mostly this will be that some
+ * comamnd doesn't include its argument.
+ *
+ * \param sfmt - The basic string and format for the specific error
+ * \param sarg - Generally the offending argument from the comamnd line.
+ *
+ * \return Nothing
+ */
+static const char use_noarg[] = "No argument for command: %s";
+#ifdef DHCPv6
+static const char use_badproto[] = "Protocol already set, %s inappropriate";
+static const char use_v4command[] = "Command not used for DHCPv6: %s";
+static const char use_v6command[] = "Command not used for DHCPv4: %s";
+#endif
+
+static void
+usage(const char *sfmt, const char *sarg) {
+
+	/* If desired print out the specific error message */
+#ifdef PRINT_SPECIFIC_CL_ERRORS
+	if (sfmt != NULL)
+		log_error(sfmt, sarg);
+#endif
+
+	log_fatal(DHCRELAY_USAGE,
+#ifdef DHCPv6
+		  isc_file_basename(progname),
+#endif
+		  isc_file_basename(progname));
 }
 
 int 
@@ -173,7 +221,6 @@ main(int argc, char **argv) {
 	isc_result_t status;
 	struct servent *ent;
 	struct server_list *sp = NULL;
-	struct interface_info *tmp = NULL;
 	char *service_local = NULL, *service_remote = NULL;
 	u_int16_t port_local = 0, port_remote = 0;
 	int no_daemon = 0, quiet = 0;
@@ -184,6 +231,12 @@ main(int argc, char **argv) {
 	int local_family_set = 0;
 #endif
 
+#ifdef OLD_LOG_NAME
+	progname = "dhcrelay";
+#else
+	progname = argv[0];
+#endif
+
 	/* Make sure that file descriptors 0(stdin), 1,(stdout), and
 	   2(stderr) are open. To do this, we assume that when we
 	   open a file the lowest available file descriptor is used. */
@@ -197,7 +250,7 @@ main(int argc, char **argv) {
 	else if (fd != -1)
 		close(fd);
 
-	openlog("dhcrelay", DHCP_LOG_OPTIONS, LOG_DAEMON);
+	openlog(isc_file_basename(progname), DHCP_LOG_OPTIONS, LOG_DAEMON);
 
 #if !defined(DEBUG)
 	setlogmask(LOG_UPTO(LOG_INFO));
@@ -223,13 +276,13 @@ main(int argc, char **argv) {
 		if (!strcmp(argv[i], "-4")) {
 #ifdef DHCPv6
 			if (local_family_set && (local_family == AF_INET6)) {
-				usage();
+				usage(use_badproto, "-4");
 			}
 			local_family_set = 1;
 			local_family = AF_INET;
 		} else if (!strcmp(argv[i], "-6")) {
 			if (local_family_set && (local_family == AF_INET)) {
-				usage();
+				usage(use_badproto, "-6");
 			}
 			local_family_set = 1;
 			local_family = AF_INET6;
@@ -241,48 +294,62 @@ main(int argc, char **argv) {
 			quiet_interface_discovery = 1;
 		} else if (!strcmp(argv[i], "-p")) {
 			if (++i == argc)
-				usage();
+				usage(use_noarg, argv[i-1]);
 			local_port = validate_port(argv[i]);
 			log_debug("binding to user-specified port %d",
 				  ntohs(local_port));
 		} else if (!strcmp(argv[i], "-c")) {
 			int hcount;
 			if (++i == argc)
-				usage();
+				usage(use_noarg, argv[i-1]);
 			hcount = atoi(argv[i]);
 			if (hcount <= 255)
 				max_hop_count= hcount;
 			else
-				usage();
+				usage("Bad hop count to -c: %s", argv[i]);
  		} else if (!strcmp(argv[i], "-i")) {
 #ifdef DHCPv6
 			if (local_family_set && (local_family == AF_INET6)) {
-				usage();
+				usage(use_v4command, argv[i]);
 			}
 			local_family_set = 1;
 			local_family = AF_INET;
 #endif
 			if (++i == argc) {
-				usage();
+				usage(use_noarg, argv[i-1]);
 			}
-			if (strlen(argv[i]) >= sizeof(tmp->name)) {
-				log_fatal("%s: interface name too long "
-					  "(is %ld)",
-					  argv[i], (long)strlen(argv[i]));
+
+			request_v4_interface(argv[i], INTERFACE_STREAMS);
+		} else if (!strcmp(argv[i], "-iu")) {
+#ifdef DHCPv6
+			if (local_family_set && (local_family == AF_INET6)) {
+				usage(use_v4command, argv[i]);
 			}
-			status = interface_allocate(&tmp, MDL);
-			if (status != ISC_R_SUCCESS) {
-				log_fatal("%s: interface_allocate: %s",
-					  argv[i],
-					  isc_result_totext(status));
+			local_family_set = 1;
+			local_family = AF_INET;
+#endif
+			if (++i == argc) {
+				usage(use_noarg, argv[i-1]);
 			}
-			strcpy(tmp->name, argv[i]);
-			interface_snorf(tmp, INTERFACE_REQUESTED);
-			interface_dereference(&tmp, MDL);
+
+			request_v4_interface(argv[i], INTERFACE_UPSTREAM);
+		} else if (!strcmp(argv[i], "-id")) {
+#ifdef DHCPv6
+			if (local_family_set && (local_family == AF_INET6)) {
+				usage(use_v4command, argv[i]);
+			}
+			local_family_set = 1;
+			local_family = AF_INET;
+#endif
+			if (++i == argc) {
+				usage(use_noarg, argv[i-1]);
+			}
+
+			request_v4_interface(argv[i], INTERFACE_DOWNSTREAM);
 		} else if (!strcmp(argv[i], "-a")) {
 #ifdef DHCPv6
 			if (local_family_set && (local_family == AF_INET6)) {
-				usage();
+				usage(use_v4command, argv[i]);
 			}
 			local_family_set = 1;
 			local_family = AF_INET;
@@ -291,13 +358,13 @@ main(int argc, char **argv) {
 		} else if (!strcmp(argv[i], "-A")) {
 #ifdef DHCPv6
 			if (local_family_set && (local_family == AF_INET6)) {
-				usage();
+				usage(use_v4command, argv[i]);
 			}
 			local_family_set = 1;
 			local_family = AF_INET;
 #endif
 			if (++i == argc)
-				usage();
+				usage(use_noarg, argv[i-1]);
 
 			dhcp_max_agent_option_packet_length = atoi(argv[i]);
 
@@ -308,13 +375,13 @@ main(int argc, char **argv) {
 		} else if (!strcmp(argv[i], "-m")) {
 #ifdef DHCPv6
 			if (local_family_set && (local_family == AF_INET6)) {
-				usage();
+				usage(use_v4command, argv[i]);
 			}
 			local_family_set = 1;
 			local_family = AF_INET;
 #endif
 			if (++i == argc)
-				usage();
+				usage(use_noarg, argv[i-1]);
 			if (!strcasecmp(argv[i], "append")) {
 				agent_relay_mode = forward_and_append;
 			} else if (!strcasecmp(argv[i], "replace")) {
@@ -324,11 +391,42 @@ main(int argc, char **argv) {
 			} else if (!strcasecmp(argv[i], "discard")) {
 				agent_relay_mode = discard;
 			} else
-				usage();
+				usage("Unknown argument to -m: %s", argv[i]);
+		} else if (!strcmp(argv [i], "-U")) {
+			if (++i == argc)
+				usage(use_noarg, argv[i-1]);
+
+			if (uplink) {
+				usage("more than one uplink (-U) specified: %s"
+				      ,argv[i]);
+			}
+
+			/* Allocate the uplink interface */
+			status = interface_allocate(&uplink, MDL);
+			if (status != ISC_R_SUCCESS) {
+				log_fatal("%s: uplink interface_allocate: %s",
+					 argv[i], isc_result_totext(status));
+			}
+		
+			if (strlen(argv[i]) >= sizeof(uplink->name)) {
+				log_fatal("%s: uplink name too long,"
+					  " it cannot exceed: %ld characters",
+					  argv[i], (long)(sizeof(uplink->name) - 1));
+			}
+
+			uplink->name[sizeof(uplink->name) - 1] = 0x00;
+			strncpy(uplink->name, argv[i],
+				sizeof(uplink->name) - 1);
+			interface_snorf(uplink, (INTERFACE_REQUESTED |
+						INTERFACE_STREAMS));
+
+			/* Turn on -a, in case they don't do so explicitly */
+			add_agent_options = 1;
+			add_rfc3527_suboption = 1;
 		} else if (!strcmp(argv[i], "-D")) {
 #ifdef DHCPv6
 			if (local_family_set && (local_family == AF_INET6)) {
-				usage();
+				usage(use_v4command, argv[i]);
 			}
 			local_family_set = 1;
 			local_family = AF_INET;
@@ -337,48 +435,48 @@ main(int argc, char **argv) {
 #ifdef DHCPv6
 		} else if (!strcmp(argv[i], "-I")) {
 			if (local_family_set && (local_family == AF_INET)) {
-				usage();
+				usage(use_v6command, argv[i]);
 			}
 			local_family_set = 1;
 			local_family = AF_INET6;
 			use_if_id = ISC_TRUE;
 		} else if (!strcmp(argv[i], "-l")) {
 			if (local_family_set && (local_family == AF_INET)) {
-				usage();
+				usage(use_v6command, argv[i]);
 			}
 			local_family_set = 1;
 			local_family = AF_INET6;
 			if (downstreams != NULL)
 				use_if_id = ISC_TRUE;
 			if (++i == argc)
-				usage();
+				usage(use_noarg, argv[i-1]);
 			sl = parse_downstream(argv[i]);
 			sl->next = downstreams;
 			downstreams = sl;
 		} else if (!strcmp(argv[i], "-u")) {
 			if (local_family_set && (local_family == AF_INET)) {
-				usage();
+				usage(use_v6command, argv[i]);
 			}
 			local_family_set = 1;
 			local_family = AF_INET6;
 			if (++i == argc)
-				usage();
+				usage(use_noarg, argv[i-1]);
 			sl = parse_upstream(argv[i]);
 			sl->next = upstreams;
 			upstreams = sl;
 		} else if (!strcmp(argv[i], "-s")) {
 			if (local_family_set && (local_family == AF_INET)) {
-				usage();
+				usage(use_v6command, argv[i]);
 			}
 			local_family_set = 1;
 			local_family = AF_INET6;
 			if (++i == argc)
-				usage();
+				usage(use_noarg, argv[i-1]);
 			dhcrelay_sub_id = argv[i];
 #endif
 		} else if (!strcmp(argv[i], "-pf")) {
 			if (++i == argc)
-				usage();
+				usage(use_noarg, argv[i-1]);
 			path_dhcrelay_pid = argv[i];
 			no_dhcrelay_pid = ISC_TRUE;
 		} else if (!strcmp(argv[i], "--no-pid")) {
@@ -388,17 +486,21 @@ main(int argc, char **argv) {
 			exit(0);
 		} else if (!strcmp(argv[i], "--help") ||
 			   !strcmp(argv[i], "-h")) {
-			log_info(DHCRELAY_USAGE);
+			log_info(DHCRELAY_USAGE,
+#ifdef DHCPv6
+				 isc_file_basename(progname),
+#endif
+				 isc_file_basename(progname));
 			exit(0);
  		} else if (argv[i][0] == '-') {
-			usage();
+			usage("Unknown command: %s", argv[i]);
  		} else {
 			struct hostent *he;
 			struct in_addr ia, *iap = NULL;
 
 #ifdef DHCPv6
 			if (local_family_set && (local_family == AF_INET6)) {
-				usage();
+				usage(use_v4command, argv[i]);
 			}
 			local_family_set = 1;
 			local_family = AF_INET;
@@ -510,7 +612,7 @@ main(int argc, char **argv) {
 		if (upstreams == NULL || downstreams == NULL) {
 			log_info("Must specify at least one lower "
 				 "and one upper interface.\n");
-			usage();
+			usage(NULL, NULL);
 		}
 
 		/* Set up the initial dhcp option universe. */
@@ -648,6 +750,11 @@ do_relay4(struct interface_info *ip, struct dhcp_packet *packet,
 
 	/* If it's a bootreply, forward it to the client. */
 	if (packet->op == BOOTREPLY) {
+		if (!(ip->flags & INTERFACE_UPSTREAM)) {
+			log_debug("Dropping reply received on %s", ip->name);
+			return;
+		}
+
 		if (!(packet->flags & htons(BOOTP_BROADCAST)) &&
 			can_unicast_without_arp(out)) {
 			to.sin_addr = packet->yiaddr;
@@ -705,8 +812,14 @@ do_relay4(struct interface_info *ip, struct dhcp_packet *packet,
 	if (out)
 		return;
 
+	if (!(ip->flags & INTERFACE_DOWNSTREAM)) {
+		log_debug("Dropping request received on %s", ip->name);
+		return;
+	}
+
 	/* Add relay agent options if indicated.   If something goes wrong,
-	   drop the packet. */
+	 * drop the packet.  Note this may set packet->giaddr if RFC3527
+	 * is enabled. */
 	if (!(length = add_relay_agent_options(ip, packet, length,
 					       ip->addresses[0])))
 		return;
@@ -948,6 +1061,7 @@ add_relay_agent_options(struct interface_info *ip, struct dhcp_packet *packet,
 	int is_dhcp = 0, mms;
 	unsigned optlen;
 	u_int8_t *op, *nextop, *sp, *max, *end_pad = NULL;
+	int adding_link_select;
 
 	/* If we're not adding agent options to packets, we can skip
 	   this. */
@@ -961,6 +1075,10 @@ add_relay_agent_options(struct interface_info *ip, struct dhcp_packet *packet,
 
 	max = ((u_int8_t *)packet) + dhcp_max_agent_option_packet_length;
 
+	/* Add link selection suboption if enabled and we're the first relay */
+	adding_link_select = (add_rfc3527_suboption
+			      && (packet->giaddr.s_addr == 0));
+
 	/* Commence processing after the cookie. */
 	sp = op = &packet->options[4];
 
@@ -1086,10 +1204,14 @@ add_relay_agent_options(struct interface_info *ip, struct dhcp_packet *packet,
 	if (ip->remote_id) {
 		if (ip->remote_id_len > 255 || ip->remote_id_len < 1)
 			log_fatal("Remote ID length %d out of range [1-255] "
-				  "on %s\n", ip->circuit_id_len, ip->name);
+				  "on %s\n", ip->remote_id_len, ip->name);
 		optlen += ip->remote_id_len + 2;    /* RAI_REMOTE_ID + len */
 	}
 
+	if (adding_link_select) {
+		optlen += 6;
+	}
+
 	/* We do not support relay option fragmenting(multiple options to
 	 * support an option data exceeding 255 bytes).
 	 */
@@ -1121,6 +1243,19 @@ add_relay_agent_options(struct interface_info *ip, struct dhcp_packet *packet,
 			memcpy(sp, ip->remote_id, ip->remote_id_len);
 			sp += ip->remote_id_len;
 		}
+
+		/* RFC3527: Use the inbound packet's interface address in
+		 * the link selection suboption and set the outbound giaddr
+		 * to the uplink address. */
+		if (adding_link_select) {
+			*sp++ = RAI_LINK_SELECT;
+			*sp++ = 4u;
+			memcpy(sp, &giaddr.s_addr, 4);
+			sp += 4;
+			packet->giaddr = uplink->addresses[0];
+			log_debug ("Adding link selection suboption"
+				   " with addr: %s", inet_ntoa(giaddr));
+		}
 	} else {
 		++agent_option_errors;
 		log_error("No room in packet (used %d of %d) "
@@ -1178,8 +1313,7 @@ parse_downstream(char *arg) {
 		*iid++ = '\0';
 	}
 	if (strlen(ifname) >= sizeof(ifp->name)) {
-		log_error("Interface name '%s' too long", ifname);
-		usage();
+		usage("Interface name '%s' too long", ifname);
 	}
 
 	/* Don't declare twice. */
@@ -1394,6 +1528,7 @@ process_up6(struct packet *packet, struct stream_list *dp) {
 	      case DHCPV6_INFORMATION_REQUEST:
 	      case DHCPV6_RELAY_FORW:
 	      case DHCPV6_LEASEQUERY:
+	      case DHCPV6_DHCPV4_QUERY:
 		log_info("Relaying %s from %s port %d going up.",
 			 dhcpv6_type_names[packet->dhcpv6_msg_type],
 			 piaddr(packet->client_addr),
@@ -1405,6 +1540,7 @@ process_up6(struct packet *packet, struct stream_list *dp) {
 	      case DHCPV6_RECONFIGURE:
 	      case DHCPV6_RELAY_REPL:
 	      case DHCPV6_LEASEQUERY_REPLY:
+	      case DHCPV6_DHCPV4_RESPONSE:
 		log_info("Discarding %s from %s port %d going up.",
 			 dhcpv6_type_names[packet->dhcpv6_msg_type],
 			 piaddr(packet->client_addr),
@@ -1623,6 +1759,7 @@ process_down6(struct packet *packet) {
 	      case DHCPV6_RECONFIGURE:
 	      case DHCPV6_RELAY_FORW:
 	      case DHCPV6_LEASEQUERY_REPLY:
+	      case DHCPV6_DHCPV4_RESPONSE:
 		log_info("Relaying %s to %s port %d down.",
 			 dhcpv6_type_names[msg->msg_type],
 			 piaddr(peer),
@@ -1638,6 +1775,7 @@ process_down6(struct packet *packet) {
 	      case DHCPV6_DECLINE:
 	      case DHCPV6_INFORMATION_REQUEST:
 	      case DHCPV6_LEASEQUERY:
+	      case DHCPV6_DHCPV4_QUERY:
 		log_info("Discarding %s to %s port %d down.",
 			 dhcpv6_type_names[msg->msg_type],
 			 piaddr(peer),
@@ -1735,3 +1873,40 @@ dhcp_set_control_state(control_object_state_t oldstate,
 
 	exit(0);
 }
+
+/*!
+ *
+ * \brief Allocate an interface as requested with a given set of flags
+ *
+ * The requested interface is allocated, its flags field is set to
+ * INTERFACE_REQUESTED OR'd with the given flags,  and then added to
+ * the list of interfaces.
+ *
+ * \param name - name of the requested interface
+ * \param flags - additional flags for the interface
+ *
+ * \return Nothing
+ */
+void request_v4_interface(const char* name, int flags) {
+        struct interface_info *tmp = NULL;
+        int len = strlen(name);
+        isc_result_t status;
+
+        if (len >= sizeof(tmp->name)) {
+                log_fatal("%s: interface name too long (is %d)", name, len);
+        }
+
+        status = interface_allocate(&tmp, MDL);
+        if (status != ISC_R_SUCCESS) {
+                log_fatal("%s: interface_allocate: %s", name,
+                          isc_result_totext(status));
+        }
+
+	log_debug("Requesting: %s as upstream: %c downstream: %c", name,
+		  (flags & INTERFACE_UPSTREAM ? 'Y' : 'N'),
+		  (flags & INTERFACE_DOWNSTREAM ? 'Y' : 'N'));
+
+        strncpy(tmp->name, name, len);
+        interface_snorf(tmp, (INTERFACE_REQUESTED | flags));
+        interface_dereference(&tmp, MDL);
+}
